@@ -20,16 +20,20 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#include "libmpdclient.h"
 #include "list.h"
 #include "charset.h"
 #include "password.h"
 #include "util.h"
 #include "status.h"
 #include "command.h"
+#include "sticker.h"
+#include "idle.h"
+#include "search.h"
 #include "mpc.h"
 #include "options.h"
 
+#include <mpd/client.h>
+
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
@@ -75,48 +79,72 @@ static struct command {
 	{"playlist",    0,   0,   0,    cmd_playlist,    "", "Print the current playlist"},
 	{"listall",     0,   -1,  2,    cmd_listall,     "[<file>]", "List all songs in the music dir"},
 	{"ls",          0,   -1,  2,    cmd_ls,          "[<directory>]", "List the contents of <directory>"},
-	{"lsplaylists", 0,   -1,  2,    cmd_lsplaylists, "", "Lists currently available playlists"},
+	{"lsplaylists", 0,   -1,  2,    cmd_lsplaylists, "", "List currently available playlists"},
 	{"load",        0,   -1,  1,    cmd_load,        "<file>", "Load <file> as a playlist"},
-	{"save",        1,   1,   0,    cmd_save,        "<file>", "Saves a playlist as <file>"},
-	{"rm",          1,   1,   0,    cmd_rm,          "<file>", "Removes a playlist"},
-	{"volume",      0,   1,   0,    cmd_volume,      "[+-]<num>", "Sets volume to <num> or adjusts by [+-]<num>"},
+	{"save",        1,   1,   0,    cmd_save,        "<file>", "Save a playlist as <file>"},
+	{"rm",          1,   1,   0,    cmd_rm,          "<file>", "Remove a playlist"},
+	{"volume",      0,   1,   0,    cmd_volume,      "[+-]<num>", "Set volume to <num> or adjusts by [+-]<num>"},
 	{"repeat",      0,   1,   0,    cmd_repeat,      "<on|off>", "Toggle repeat mode, or specify state"},
 	{"random",      0,   1,   0,    cmd_random,      "<on|off>", "Toggle random mode, or specify state"},
 	{"single",      0,   1,   0,    cmd_single,      "<on|off>", "Toggle single mode, or specify state"},
 	{"consume",     0,   1,   0,    cmd_consume,     "<on|off>", "Toggle consume mode, or specify state"},
 	{"search",      2,   -1,  0,    cmd_search,      "<type> <query>", "Search for a song"},
 	{"find",        2,   -1,  0,    cmd_find,        "<type> <query>", "Find a song (exact match)"},
+	{"findadd", 2, -1, 0, cmd_findadd, "<type> <query>",
+	 "Find songs and add them to the current playlist"},
 	{"list",        1,   -1,  0,    cmd_list,        "<type> [<type> <query>]", "Show all tags of <type>"},
 	{"crossfade",   0,   1,   0,    cmd_crossfade,   "[<seconds>]", "Set and display crossfade settings"},
-	{"update",      0,   -1,  2,    cmd_update,      "[<path>]", "Scans music directory for updates"},
-	{"stats",       0,   -1,  0,    cmd_stats,       "", "Displays statistics about MPD"},
-	{"version",     0,   0,   0,    cmd_version,     "", "Reports version of MPD"},
+	{"update",      0,   -1,  2,    cmd_update,      "[<path>]", "Scan music directory for updates"},
+#if defined(LIBMPDCLIENT_CHECK_VERSION)
+#if LIBMPDCLIENT_CHECK_VERSION(2,1,0)
+	{"sticker",     1,   -1,  0,    cmd_sticker,     "<uri> <get|set|list|del> <args..>", "Sticker management"},
+#endif
+#endif
+	{"stats",       0,   -1,  0,    cmd_stats,       "", "Display statistics about MPD"},
+	{"version",     0,   0,   0,    cmd_version,     "", "Report version of MPD"},
 	/* loadtab, lstab, and tab used for completion-scripting only */
 	{"loadtab",     0,   1,   0,    cmd_loadtab,     "<directory>", NULL},
 	{"lstab",       0,   1,   0,    cmd_lstab,       "<directory>", NULL},
 	{"tab",         0,   1,   0,    cmd_tab,         "<path>", NULL},
 	/* status was added for pedantic reasons */
 	{"status",      0,   -1,  0,    cmd_status,      "", NULL},
-	{ "idle", 0, 0, 0, cmd_idle, "", "Idle until an event occurs" },
+	{ "idle", 0, -1, 0, cmd_idle, "[events]",
+	  "Idle until an event occurs" },
+	{ "idleloop", 0, -1, 0, cmd_idleloop, "[events]",
+	  "Continuously idle until an event occurs" },
+	{ "replaygain", 0, -1, 0, cmd_replaygain, "[off|track|ablum]",
+	  "Set or display the replay gain mode" },
+
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
 	{ .command = NULL }
 };
 
+static void
+print_usage(FILE * outfp, char * progname)
+{
+	fprintf(outfp,"Usage: %s [options] <command> [<arguments>]\n"
+		"mpc version: "VERSION"\n",progname);
+}
+
 static int print_help(char * progname, char * command)
 {
 	int i, max = 0;
-	int ret = EXIT_FAILURE;
-	FILE *outfp = stderr;
 
-	if (command) {
-		if (!strcmp(command,"help")) {
-			outfp = stdout;
-			ret = EXIT_SUCCESS;
-		} else
-			fprintf(outfp,"unknown command \"%s\"\n",command);
+	if (command && strcmp(command, "help")) {
+		fprintf(stderr,"unknown command \"%s\"\n",command);
+		print_usage(stderr, progname);
+		fprintf(stderr,"See man 1 mpc or use 'mpc help' for more help.\n");
+		return EXIT_FAILURE;
 	}
-	fprintf(outfp,"Usage: %s <command> [command args]...\n"
-		"mpc version: "VERSION"\n",progname);
+
+	print_usage(stdout, progname);
+	printf("\n");
+
+	printf("Options:\n");
+	print_option_help();
+	printf("\n");
+
+	printf("Commands:\n");
 
 	for (i=0; mpc_table[i].command; ++i) {
 		if (mpc_table[i].help) {
@@ -126,7 +154,7 @@ static int print_help(char * progname, char * command)
 		}
 	}
 
-	fprintf(outfp,	"%s %*s  Displays status\n",progname,max," ");
+	printf("  %s %*s  Display status\n",progname,max," ");
 
 	for (i=0; mpc_table[i].command; ++i) {
 		int spaces;
@@ -136,58 +164,31 @@ static int print_help(char * progname, char * command)
 		spaces = max-(strlen(mpc_table[i].command)+strlen(mpc_table[i].usage));
 		spaces += !spaces ? 0 : 1;
 
-		fprintf(outfp,"%s %s %s%*s%s\n",progname,
+		printf("  %s %s %s%*s%s\n",progname,
 			mpc_table[i].command,mpc_table[i].usage,
 			spaces," ",mpc_table[i].help);
 
 	}
-	fprintf(outfp,"For more information about these and other "
-			"options look at man 1 mpc\n");
-	return ret;
+	printf("\nSee man 1 mpc for more information about mpc commands and options\n");
+	return EXIT_SUCCESS;
 }
 
-static mpd_Connection * setup_connection (void)
+static struct mpd_connection *
+setup_connection(void)
 {
-	const char * host = DEFAULT_HOST;
-	const char * port = DEFAULT_PORT;
-	int iport;
-	char * test;
-	int port_env = 0;
-	int host_env = 0;
-	int password_len= 0;
-	int parsed_len = 0;
-	mpd_Connection * conn;
+	struct mpd_connection *conn;
 
-	if((test = getenv("MPD_HOST"))) {
-		host =test;
-		host_env = 1;
-	}
-
-	if((test = getenv("MPD_PORT"))) {
-		port = test;
-		port_env = 1;
-	}
-
-	iport = strtol(port,&test,10);
-
-	if(iport<0 || *test!='\0') {
-		fprintf(stderr,"MPD_PORT \"%s\" is not a positive integer\n",
-				port);
+	conn = mpd_connection_new(options.host, options.port, 0);
+	if (conn == NULL) {
+		fputs("Out of memory\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
-	parse_password(host, &password_len, &parsed_len);
+	if (mpd_connection_get_error(conn) != MPD_ERROR_SUCCESS)
+		printErrorAndExit(conn);
 
-	conn = mpd_newConnection(host+parsed_len,iport,10);
-
-	if(conn->error && (!port_env || !host_env))
-		fprintf(stderr,"MPD_HOST and/or MPD_PORT environment variables"
-			" are not set\n");
-
-	printErrorAndExit(conn);
-
-	if(password_len)
-		send_password (host, password_len, conn);
+	if(options.password)
+		send_password(options.password, conn);
 
 	return conn;
 }
@@ -235,18 +236,16 @@ static int
 run(const struct command *command, int argc, char **array)
 {
 	int ret;
-	mpd_Connection *conn;
+	struct mpd_connection *conn;
 
 	conn = setup_connection();
 
 	ret = command->handler(argc, array, conn);
-	if (ret != 0) {
-		struct mpc_option* nostatus = get_option("no-status");
-		if (!nostatus->set)
-			print_status(conn);
+	if (ret != 0 && options.verbosity > V_QUIET) {
+		print_status(conn);
 	}
 
-	mpd_closeConnection(conn);
+	mpd_connection_free(conn);
 	return (ret >= 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
@@ -256,8 +255,7 @@ int main(int argc, char ** argv)
 	const char *command_name;
 	struct command *command;
 
-	if(parse_options(&argc, argv) < 0)
-		return print_help(argv[0],NULL);
+	parse_options(&argc, argv);
 
 	/* parse command and arguments */
 
@@ -280,8 +278,7 @@ int main(int argc, char ** argv)
 
 	/* initialization */
 
-	charset_init(command->pipe >= 0 || isatty(STDIN_FILENO),
-		     isatty(STDOUT_FILENO));
+	charset_init(true, true);
 
 	/* run */
 
