@@ -25,23 +25,10 @@
 
 #include "config.h"
 
-#include "libmpdclient.h"
-#define STRING_LENGTH	(8192)
-
 #define STDIN_SYMBOL	"-"
 
-#ifndef DEFAULT_HOST
-#  define DEFAULT_HOST "localhost"
-#endif /* DEFAULT_HOST */
+struct mpd_connection;
 
-#ifndef DEFAULT_PORT
-#  define DEFAULT_PORT "6600"
-#endif /* DEFAULT_PORT */
-
-#ifndef MIN_COLUMNS
-#  define MIN_COLUMNS	80
-#endif /* MIN_COLUMNS */
-
-typedef int (* cmdhandler)(int argc, char ** argv, mpd_Connection * conn);
+typedef int (*cmdhandler)(int argc, char **argv, struct mpd_connection *conn);
 
 #endif /* MPC_H */
