@@ -22,7 +22,10 @@
 
 #ifndef PASSWORD_H
 #define PASSWORD_H
-#include "libmpdclient.h"
-void parse_password (const char * host, int * pass_len, int * parsed_len);
-void send_password (const char * host, int pass_len, mpd_Connection * conn);
+
+struct mpd_connection;
+
+void
+send_password(const char *password, struct mpd_connection *conn);
+
 #endif /* PASSWORD_H */
