@@ -21,12 +21,12 @@
 */
 
 #include "util.h"
-#include "libmpdclient.h"
 #include "charset.h"
 #include "list.h"
-#include "mpc.h"
 #include "options.h"
 
+#include <mpd/client.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -34,25 +34,34 @@
 #include <assert.h>
 #include <sys/param.h>
 
-void printErrorAndExit(mpd_Connection * conn)
+void
+printErrorAndExit(struct mpd_connection *conn)
 {
-	if(conn->error) {
-		fprintf(stderr,"error: %s\n",charset_from_utf8(conn->errorStr));
-		mpd_closeConnection(conn);
-		exit(EXIT_FAILURE);
-	}
+	const char *message;
+
+	assert(mpd_connection_get_error(conn) != MPD_ERROR_SUCCESS);
+
+	message = mpd_connection_get_error_message(conn);
+	if (mpd_connection_get_error(conn) == MPD_ERROR_SERVER)
+		/* messages received from the server are UTF-8; the
+		   rest is either US-ASCII or locale */
+		message = charset_from_utf8(message);
+
+	fprintf(stderr, "error: %s\n", message);
+	mpd_connection_free(conn);
+	exit(EXIT_FAILURE);
 }
 
 int stdinToArgArray(char *** array)
 {
 	List * list = makeList(NULL);
 	ListNode * node;
-	char buffer[STRING_LENGTH+1];
+	char buffer[4096];
 	int size;
 	int i;
 	char * sp;
 
-	while(fgets(buffer,STRING_LENGTH,stdin)) {
+	while(fgets(buffer, sizeof(buffer), stdin)) {
 		if((sp = strchr(buffer,'\n'))) *sp = '\0';
 		insertInListWithoutKey(list,strdup(buffer));
 	}
@@ -109,51 +118,6 @@ int get_boolean (const char * arg)
 	return -1;
 }
 
-int get_search_type(const char * arg)
-{
-	int i;
-
-	for (i = 0; i < MPD_TAG_NUM_OF_ITEM_TYPES; i++) {
-		if (strcasecmp(mpdTagItemKeys[i], arg) == 0)
-			return i;
-	}
-
-	fprintf(stderr, "\"%s\" is not a valid search type: <", arg);
-
-	for (i = 0; i < MPD_TAG_NUM_OF_ITEM_TYPES; i++) {
-		fprintf(stderr, "%c%s%s", tolower(mpdTagItemKeys[i][0]),
-		                          mpdTagItemKeys[i]+1,
-		                          (mpdTagItemKeys[i+1] ? "|" : ">\n"));
-	}
-
-	return -1;
-}
-
-int get_constraints(int argc, char **argv, Constraint **constraints)
-{
-	int numconstraints = 0;
-	int type;
-	int i;
-
-	assert(argc > 0 && argc % 2 == 0);
-
-	*constraints = malloc(sizeof(Constraint)*argc/2);
-
-	for (i = 0; i < argc; i += 2) {
-		type = get_search_type(argv[i]);
-		if (type < 0) {
-			free(*constraints);
-			return -1;
-		}
-
-		(*constraints)[numconstraints].type = type;
-		(*constraints)[numconstraints].query = argv[i+1];
-		numconstraints++;
-	}
-
-	return numconstraints;
-}
-
 /* note - return value is success; the parsed int itself is in ret */
 
 int parse_int(const char * str, int * ret)
@@ -259,6 +223,46 @@ static const char * skipFormatting(const char * p) {
 	return p;
 }
 
+static const char *
+song_value(const struct mpd_song *song, const char *name)
+{
+	static char buffer[10];
+	const char *value;
+
+	if (strcmp(name, "file") == 0)
+		value = mpd_song_get_uri(song);
+	else if (strcmp(name, "time") == 0) {
+		unsigned duration = mpd_song_get_duration(song);
+
+		if (duration > 0) {
+			snprintf(buffer, sizeof(buffer), "%d:%02d",
+				 duration / 60, duration % 60);
+			value = buffer;
+		} else
+			value = NULL;
+	} else if (strcmp(name, "position") == 0) {
+	        unsigned pos = mpd_song_get_pos(song);
+		snprintf(buffer, sizeof(buffer), "%d", pos+1);
+		value = buffer;
+	} else if (strcmp(name, "id") == 0) {
+		snprintf(buffer, sizeof(buffer), "%u", mpd_song_get_id(song));
+		value = buffer;
+	} else {
+		enum mpd_tag_type tag_type = mpd_tag_name_iparse(name);
+		if (tag_type == MPD_TAG_UNKNOWN)
+			return NULL;
+
+		value = mpd_song_get_tag(song, tag_type, 0);
+	}
+
+	if (value != NULL)
+		value = charset_from_utf8(value);
+	else
+		value = "";
+
+	return value;
+}
+
 /* this is a little ugly... */
 static char *
 songToFormatedString(struct mpd_song *song,
@@ -267,9 +271,9 @@ songToFormatedString(struct mpd_song *song,
 	char * ret = NULL;
 	const char *p, *end;
 	const char *temp;
+	char name[32];
 	int length;
 	int found = 0;
-	int labelFound = 0;
 
 	/* we won't mess up format, we promise... */
 	for (p = format; *p != '\0'; )
@@ -367,63 +371,28 @@ songToFormatedString(struct mpd_song *song,
 		}
 		length = end - p + 1;
 
-		labelFound = 0;
-
 		if (*end != '%') {
-			length--;
-		} else if (strncmp("%file%", p, length) == 0) {
-			temp = charset_from_utf8(song->file);
-		} else if (strncmp("%artist%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->artist ? charset_from_utf8(song->artist) : NULL;
-		} else if (strncmp("%album%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->album ? charset_from_utf8(song->album) : NULL;
-		} else if (strncmp("%track%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->track ? charset_from_utf8(song->track) : NULL;
-		} else if (strncmp("%title%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->title ? charset_from_utf8(song->title) : NULL;
-		} else if (strncmp("%name%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->name ? charset_from_utf8(song->name) : NULL;
-		} else if (strncmp("%date%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->date ? charset_from_utf8(song->date) : NULL;
-		} else if (strncmp("%genre%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->genre ? charset_from_utf8(song->genre) : NULL;
-		} else if (strncmp("%composer%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->composer ? charset_from_utf8(song->composer) : NULL;
-		} else if (strncmp("%performer%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->performer ? charset_from_utf8(song->performer) : NULL;
-		} else if (strncmp("%disc%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->disc ? charset_from_utf8(song->disc) : NULL;
-		} else if (strncmp("%comment%", p, length) == 0) {
-			labelFound = 1;
-			temp = song->comment ? charset_from_utf8(song->comment) : NULL;
-		} else if (strncmp("%time%", p, length) == 0) {
-			labelFound = 1;
-			if (song->time != MPD_SONG_NO_TIME) {
-				char s[10];
-				snprintf(s, 9, "%d:%02d", song->time / 60, 
-				                          song->time % 60);
-				/* nasty hack to use static buffer */
-				temp = charset_from_utf8(s);
-			}
+			ret = appendToString(ret, p, length - 1);
+			p += length - 1;
+			continue;
 		}
 
-		if( temp == NULL && !labelFound ) {
+		if (length > (int)sizeof(name)) {
 			ret = appendToString(ret, p, length);
+			p += length;
+			continue;
 		}
-		else if( temp != NULL ) {
-			found = 1;
+
+		memcpy(name, p + 1, length - 2);
+		name[length - 2] = 0;
+
+		temp = song_value(song, name);
+		if (temp != NULL) {
+			if (*temp != 0)
+				found = 1;
 			ret = appendToString(ret, temp, strlen(temp));
-		}
+		} else
+			ret = appendToString(ret, p, length);
 
 		/* advance past the specifier */
 		p += length;
@@ -443,30 +412,21 @@ static void print_formatted_song(struct mpd_song *song, const char * format)
 	}
 }
 
-#define DEFAULT_FORMAT "[%name%: &[%artist% - ]%title%]|%name%|[%artist% - ]%title%|%file%"
-
 void pretty_print_song(struct mpd_song *song)
 {
-	/* was a format string specified? */
-	if (get_option("format")->set)
-	{
-		print_formatted_song(song, get_option("format")->value);
-	}
-	/* just do something pretty */
-	else
-	{
-		print_formatted_song(song, DEFAULT_FORMAT);
-	}
+	print_formatted_song(song, options.format);
 }
 
-void print_filenames(mpd_Connection *conn)
+void
+print_filenames(struct mpd_connection *conn)
 {
-	mpd_InfoEntity *entity;
+	struct mpd_song *song;
 
-	while ((entity = mpd_getNextInfoEntity(conn))) {
-		printErrorAndExit(conn);
-		if (entity->type == MPD_INFO_ENTITY_TYPE_SONG)
-			printf("%s\n", charset_from_utf8(entity->info.song->file));
-		mpd_freeInfoEntity(entity);
+	while ((song = mpd_recv_song(conn)) != NULL) {
+		printf("%s\n", charset_from_utf8(mpd_song_get_uri(song)));
+		mpd_song_free(song);
 	}
+
+	if (mpd_connection_get_error(conn) != MPD_ERROR_SUCCESS)
+		printErrorAndExit(conn);
 }
